//	DataStreams.h
//	Application creates an object of one of these classes and 
//	passes to Datatypes.lib functions.
//	Or application can define its own subclass of DStream for 
//	things like network connections.

#pragma once

#include <File.h>
#include <OS.h>
#include <stdlib.h>

#include <DStream.h>


#pragma export on

class DFileStream :
	public DStream
{
		BFile *					fFile;
		ssize_t					fOffset;

public:
								DFileStream(
									BFile *			file,	//	deleted by stream
									off_t			offset = 0);
								~DFileStream();

		ssize_t					Read(
									void *			buffer,
									ssize_t &		size);
		ssize_t					Write(
									const void *	buffer,
									ssize_t			size);
		ssize_t					Seek(
									off_t			position);
		off_t					Position();
		off_t					Size();
		ssize_t					SetSize(
									off_t			size);
};


class DMallocStream :
	public DStream
{
		void *					fPtr;
		ssize_t					fPhysSize;
		ssize_t					fLogSize;
		ssize_t					fLogPos;
		ssize_t					fOffset;

public:
								DMallocStream(
									void *			ptr = NULL,	//	deleted by stream
									ssize_t			size = 0,
									ssize_t			offset = 0);
								~DMallocStream();

		ssize_t					Read(
									void *			buffer,
									ssize_t &		size);
		ssize_t					Write(
									const void *	buffer,
									ssize_t			size);
		ssize_t					Seek(
									off_t			position);
		off_t					Position();
		off_t					Size();
		ssize_t					SetSize(
									off_t			size);

virtual	const void *			Peek(
									ssize_t &		outOffset,
									ssize_t &		outSize);
};


class DAreaStream :
	public DStream
{
		area_id					fArea;
		void *					fPtr;
		ssize_t					fPhysSize;
		ssize_t					fLogSize;
		ssize_t					fLogPos;
		ssize_t					fOffset;

public:
								DAreaStream(
									area_id			area = 0,	//	released by stream
									ssize_t			size = 0,		//	needed because area has size multiple of page
									ssize_t			offset = 0);
								~DAreaStream();

		ssize_t					Read(
									void *			buffer,
									ssize_t &		size);
		ssize_t					Write(
									const void *	buffer,
									ssize_t			size);
		ssize_t					Seek(
									off_t			position);
		off_t					Position();
		off_t					Size();
		ssize_t					SetSize(
									off_t			size);


virtual	const void *			Peek(
									ssize_t &		outOffset,
									ssize_t &		outSize);
};

#pragma export reset

